@@ -9,5 +9,7 @@ from .frame import Frame
 from .sample_from_obs import *
 from .split_utils import *
 from .stats import *
-from .time_freq_utils import *
-from .unit_utils import *
\ No newline at end of file
+from .unit_utils import *
+from .frame_utils import *
+from .normalize import *
+from .dedrift import *
\ No newline at end of file
