@@ -1,5 +1,7 @@
 import sys
 import os.path
+import copy
+
 import numpy as np
 import matplotlib.pyplot as plt
 try:
@@ -17,6 +19,7 @@ from . import waterfall_utils
 from . import distributions
 from . import sample_from_obs
 from . import unit_utils
+from . import frame_utils
 
 from .funcs import paths
 from .funcs import t_profiles
@@ -35,11 +38,12 @@ class Frame(object):
                  waterfall=None,
                  fchans=None,
                  tchans=None,
-                 df=None,
-                 dt=None,
-                 fch1=8*u.GHz,
+                 df=2.7939677238464355*u.Hz,
+                 dt=18.253611008*u.s,
+                 fch1=6*u.GHz,
                  ascending=False,
-                 data=None):
+                 data=None,
+                 **kwargs):
         """
         Initialize a Frame object either from an existing .fil/.h5 file or
         from frame resolution / size.
@@ -48,9 +52,14 @@ class Frame(object):
         filename or the Waterfall object into the waterfall keyword.
 
         Otherwise, you can initialize a frame by specifying the parameters
-        fchans, tchans, df, dt, and potentially fch1, if it's important to
-        specify frequencies (8*u.GHz is an arbitrary but reasonable choice
-        otherwise). The `data` keyword is only necessary if you are also
+        fchans, tchans, df, dt, and even fch1, if it's important to
+        specify frequencies (8 GHz is an arbitrary but reasonable choice
+        otherwise). Note that the frame resolutions df and dt are given 
+        defaults based on the Breakthrough Listen high frequency resolution
+        data product -- be sure to change these if you are working with 
+        different kinds of data.
+        
+        The `data` keyword is only necessary if you are also
         preloading data that matches your specified frame dimensions and
         resolutions.
 
@@ -78,23 +87,30 @@ class Frame(object):
             determined by observational parameters.
         data : ndarray, optional
             2D array of intensities to preload into frame
+        **kwargs
+            For convenience, the `shape` keyword can be used in place of individually
+            setting `fchans` and `tchans`, so that :code:`shape=(tchans, fchans)`.
         """
-        if None not in [fchans, tchans, df, dt, fch1]:
+        if None not in [fchans, tchans] or 'shape' in kwargs or data is not None:
             self.waterfall = None
 
             # Need to address this and come up with a meaningful header
             self.header = None
-            self.fchans = int(unit_utils.get_value(fchans, u.pixel))
             
-            self.ascending = ascending
             self.df = unit_utils.get_value(abs(df), u.Hz)
-            self.fch1 = unit_utils.get_value(fch1, u.Hz)
-
-            self.tchans = int(unit_utils.get_value(tchans, u.pixel))
             self.dt = unit_utils.get_value(dt, u.s)
-
-            self.shape = (self.tchans, self.fchans)
-
+            self.fch1 = unit_utils.get_value(fch1, u.Hz)
+            self.ascending = ascending
+            
+            if 'shape' in kwargs:
+                (self.tchans, self.fchans) = self.shape = kwargs['shape']
+            elif data is not None:
+                (self.tchans, self.fchans) = self.shape = data.shape
+            else:
+                self.fchans = int(unit_utils.get_value(fchans, u.pixel))
+                self.tchans = int(unit_utils.get_value(tchans, u.pixel))
+                self.shape = (self.tchans, self.fchans)
+            
             if data is not None:
                 assert data.shape == self.shape
                 self.data = np.copy(data)
@@ -110,11 +126,13 @@ class Frame(object):
                 sys.exit('Invalid data file!')
             self.header = self.waterfall.header
             self.tchans, _, self.fchans = self.waterfall.container.selection_shape
+            self.shape = (self.tchans, self.fchans)
 
             # Frequency values are saved in MHz in waterfall files
-            self.ascending = (self.waterfall.header['foff'] > 0)
             self.df = unit_utils.cast_value(abs(self.waterfall.header['foff']),
                                             u.MHz).to(u.Hz).value
+            self.dt = unit_utils.get_value(self.waterfall.header['tsamp'], u.s)
+            self.ascending = (self.waterfall.header['foff'] > 0)
             if self.ascending:
                 self.fch1 = self.waterfall.container.f_start
             else:
@@ -127,17 +145,16 @@ class Frame(object):
             self.data = waterfall_utils.get_data(self.waterfall)
             if not self.ascending:
                 self.data = self.data[:, ::-1]
-
-            self.dt = unit_utils.get_value(self.waterfall.header['tsamp'], u.s)
-
-            self.shape = (self.tchans, self.fchans)
         else:
-            raise ValueError('Frame must be provided dimensions or an \
-                              existing filterbank file.')
+            raise ValueError(f'Frame must be provided dimensions or an '
+                             f'existing filterbank file.')
             
         # Degrees of freedom for chi-squared radiometer noise
         # 2 polarizations, real and imaginary components -> 4
         self.chi2_df = 4 * round(self.df * self.dt)
+        
+        # Calculate unit drift rate (pixel over pixel drift)
+        self.unit_drift_rate = self.df / self.dt
 
         # Shared creation of ranges
         self._update_fs()
@@ -150,18 +167,50 @@ class Frame(object):
         self.metadata = {}
 
     @classmethod
-    def from_data(cls, df, dt, fch1, ascending, data):
+    def from_data(cls, df, dt, fch1, ascending, data, metadata={}, waterfall=None):
         """
-        Instantiate Frame using a data array.
+        Initialize Frame more directly from 2D numpy array of data.
+        
+        Parameters
+        ----------
+        df : astropy.Quantity
+            Frequency resolution (e.g. in u.Hz)
+        dt : astropy.Quantity
+            Time resolution (e.g. in u.s)
+        fch1 : astropy.Quantity
+            Frequency of channel 1, as in filterbank file headers (e.g. in u.Hz).
+            If ascending=True, fch1 is the minimum frequency; if ascending=False 
+            (default), fch1 is the maximum frequency.
+        ascending : bool
+            Specify whether frequencies should be in ascending order, so that 
+            fch1 is the minimum frequency. Default is False, for which fch1
+            is the maximum frequency. This is overwritten if a waterfall
+            object is provided, where ascending will be automatically 
+            determined by observational parameters.
+        data : ndarray
+            2D array of intensities to preload into frame
+        metadata : dict, optional
+            Dictionary of features associated with the frame
+        waterfall : Waterfall, optional
+            Associated Waterfall object if data is derived from another frame object 
+            (accessed via frame.get_waterfall()) or a blimpy waterfall object
+            
+        Returns
+        -------
+        frame : Frame
+            Frame object with preloaded data
         """
         tchans, fchans = data.shape
-        return cls(fchans=fchans,
-                   tchans=tchans,
-                   df=df,
-                   dt=dt,
-                   fch1=fch1,
-                   ascending=ascending,
-                   data=data)
+        frame = cls(fchans=fchans,
+                    tchans=tchans,
+                    df=df,
+                    dt=dt,
+                    fch1=fch1,
+                    ascending=ascending,
+                    data=data)
+        frame.set_metadata(metadata)
+        frame.waterfall = copy.deepcopy(waterfall)
+        return frame
 
     @classmethod
     def from_waterfall(cls, waterfall):
@@ -169,17 +218,103 @@ class Frame(object):
         Instantiate Frame using a filterbank file or blimpy Waterfall object.
         """
         return cls(waterfall=waterfall)
+    
+    @classmethod
+    def from_backend_params(cls,
+                            fchans=None,
+                            obs_length=300, 
+                            sample_rate=3e9, 
+                            num_branches=1024,
+                            fftlength=1048576,
+                            int_factor=51,
+                            fch1=6*u.GHz,
+                            ascending=False,
+                            data=None):
+        """
+        Create frame based on backend / software related parameters.
+        Either `fchans` or `data` must be provided to get number of frequency
+        channels to create. If a 2D numpy array for `data` is provided, `fchans`
+        will be inferred. The parameter `int_factor` must still be provided 
+        to determine `tchans`; there is a check that the data dimensions also match.
+        Since multiple `int_factor` values may correspond to the same `tchans`, 
+        for clarity we do not infer `int_factor` just from the dimensions of the data.
+        
+        Parameters
+        ----------
+        fchans : int, optional
+            Number of frequency samples. Should be provided if `data` is None.
+        obs_length : float, optional
+            Length of observation in seconds
+        sample_rate : float, optional
+            Physical sample rate, in Hz, for collecting real voltage data
+        num_branches : int, optional
+            Number of PFB branches. Note that this corresponds to `num_branches / 2` coarse channels.
+        fftlength : int, optional
+            FFT length to be used in fine channelization
+        int_factor : int, optional
+            Integration factor used in fine channelization. Determines tchans.
+        fch1 : astropy.Quantity, optional
+            Frequency of channel 1, as in filterbank file headers (e.g. in u.Hz).
+            If ascending=True, fch1 is the minimum frequency; if ascending=False 
+            (default), fch1 is the maximum frequency.
+        ascending : bool, optional
+            Specify whether frequencies should be in ascending order, so that 
+            fch1 is the minimum frequency. Default is False, for which fch1
+            is the maximum frequency.
+        data : ndarray, optional
+            2D array of intensities to preload into frame. If provided, `fchans`
+            will be inferred from this. 
+            
+        Returns
+        -------
+        frame : Frame
+            Frame object with appropriate dimensions.
+        """
+        chan_bw = sample_rate / num_branches
+        df = chan_bw / fftlength
+        
+        if data is not None:
+            tchans, fchans = data.shape
+        elif fchans is None:
+            raise ValueError("Value not given for fchans")
+            
+        param_dict = params_from_backend(obs_length=obs_length, 
+                                         sample_rate=sample_rate, 
+                                         num_branches=num_branches,
+                                         fftlength=fftlength,
+                                         int_factor=int_factor)
+        if data is not None:
+            assert param_dict['tchans'] == tchans
+        
+        frame = cls(fchans=fchans,
+                    **param_dict,
+                    fch1=fch1,
+                    ascending=ascending,
+                    data=data)
+        return frame
+        
+    def copy(self):
+        """
+        Return identical copy of frame.
+        """
+        c_frame = copy.deepcopy(self)
+        # Note that since the __getstate__ function is overwritten, we need to
+        # add back the waterfall object.
+        waterfall = self.get_waterfall()
+        if waterfall is not None:
+            c_frame.waterfall = copy.deepcopy(waterfall)
+        return c_frame
 
     def __getstate__(self):
         # Exclude waterfall Waterfall object from pickle, since it uses open threads, which
-        # can't be pickled
+        # can't be pickled -- note that this affects copy!
         state = self.__dict__.copy()
         state['waterfall'] = None
         return state
 
     def _update_fs(self):
         """
-        Calculates and updates an array of frequencies represented in the
+        Calculate and update an array of frequencies represented in the
         frame.
         """
         # Normally, self.ascending will be False; filterbank convention is decreasing freqs
@@ -201,7 +336,7 @@ class Frame(object):
 
     def _update_ts(self):
         """
-        Calculates and updates an array of times represented in the frame.
+        Calculate and update an array of times represented in the frame.
         """
         self.ts = unit_utils.get_value(np.linspace(0,
                                                    self.tchans * self.dt,
@@ -211,7 +346,7 @@ class Frame(object):
 
     def zero_data(self):
         """
-        Resets data to a numpy array of zeros.
+        Reset data to a numpy array of zeros.
         """
         self.data = np.zeros(self.shape)
         self.noise_mean = self.noise_std = 0
@@ -230,7 +365,7 @@ class Frame(object):
 
     def _update_noise_frame_stats(self):
         """
-        Calculates and updates basic noise statistics (mean and standard
+        Calculate and update basic noise statistics (mean and standard
         deviation) of the frame, using sigma clipping to strip outliers.
         """
         clipped_data = sigma_clip(self.data,
@@ -245,7 +380,7 @@ class Frame(object):
                   x_min=None,
                   noise_type='chi2'):
         """
-        By default, synthesizes radiometer noise based on a chi-squared
+        By default, synthesize radiometer noise based on a chi-squared
         distribution. Alternately, can generate pure Gaussian noise.
         
         Specifying noise_type='chi2' will only use x_mean,
@@ -294,7 +429,7 @@ class Frame(object):
                            share_index=True,
                            noise_type='chi2'):
         """
-        By default, synthesizes radiometer noise based on a chi-squared
+        By default, synthesize radiometer noise based on a chi-squared
         distribution. Alternately, can generate pure Gaussian noise.
         
         If no arrays are specified from which to sample, noise
@@ -309,7 +444,7 @@ class Frame(object):
 
         Note: this method will attempt to scale the noise parameters to match
         self.dt and self.df. This assumes that the observation data products
-        are *not* normalized by the FFT length used to contstruct them.
+        are *not* normalized by the FFT length used to construct them.
 
         Parameters
         ----------
@@ -402,17 +537,19 @@ class Frame(object):
                    path,
                    t_profile,
                    f_profile,
-                   bp_profile,
+                   bp_profile=None,
                    bounding_f_range=None,
                    integrate_path=False,
                    integrate_t_profile=False,
                    integrate_f_profile=False,
+                   doppler_smearing=False,
                    t_subsamples=10,
-                   f_subsamples=10):
+                   f_subsamples=10,
+                   smearing_subsamples=10):
         """
-        Generates synthetic signal.
+        Generate synthetic signal.
 
-        Adds a synethic signal using given path in time-frequency domain and
+        Add a synethic signal using given path in time-frequency domain and
         brightness profiles in time and frequency directions.
 
         Parameters
@@ -429,7 +566,7 @@ class Frame(object):
             Frequency profile: function in frequency that returns an intensity
             (scalar), relative to the signal frequency within a time sample.
             Note that unlike the other parameters, this must be a function
-        bp_profile : function, np.ndarray, list, float
+        bp_profile : function, np.ndarray, list, float, optional
             Bandpass profile: function in frequency that returns a relative
             intensity (scalar, between 0 and 1), or provided array or single
             value of relative intensities at each frequency sample
@@ -452,13 +589,21 @@ class Frame(object):
         integrate_f_profile : bool, optional
             Option to integrate f_profile in the frequency direction. Makes
             `f_subsamples` calculations per time sample.
+        doppler_smearing : bool, optional
+            Option to numerically "Doppler smear" spectral power over 
+            frequency bins. At time t, averages `smearing_subsamples` copies of
+            the signal centered at evenly spaced center frequencies between 
+            times t and t+1. This causes the effective drop in power when 
+            the signal crosses multiple bins.
         t_subsamples : int, optional
             Number of bins for integration in the time direction, using
-            Riemann sums
+            Riemann sums. Default is 10.
         f_subsamples : int, optional
             Number of bins for integration in the frequency direction, using
-            Riemann sums
-
+            Riemann sums. Default is 10.
+        smearing_subsamples : int, optional
+            Number of steps for averaging evenly spaced copies of the signal 
+            between center frequencies at times t and t+1. Default is 10.
         Returns
         -------
         signal : ndarray
@@ -496,7 +641,7 @@ class Frame(object):
         >>> %matplotlib inline
         >>> import matplotlib.pyplot as plt
         >>> fig = plt.figure(figsize=(10, 6))
-        >>> frame.render()
+        >>> frame.plot()
         >>> plt.savefig('image.png', bbox_inches='tight')
         >>> plt.show()
 
@@ -516,8 +661,9 @@ class Frame(object):
             restricted_fchans = len(restricted_fs)
             restricted_fs = np.linspace(f0,
                                         f0 + restricted_fchans * self.df,
-                                        restricted_fchans * f_subsamples)
-        ff, tt = np.meshgrid(restricted_fs, self.ts)
+                                        restricted_fchans * f_subsamples,
+                                        endpoint=False)
+        ff, _ = np.meshgrid(restricted_fs, self.ts)
 
         # Handle t_profile
         if callable(t_profile):
@@ -526,7 +672,8 @@ class Frame(object):
             if integrate_t_profile:
                 new_ts = np.linspace(0,
                                      self.tchans * self.dt,
-                                     self.tchans * t_subsamples)
+                                     self.tchans * t_subsamples,
+                                     endpoint=False)
                 y = t_profile(new_ts)
                 if not isinstance(y, np.ndarray):
                     y = np.repeat(y, self.tchans * t_subsamples)
@@ -545,37 +692,55 @@ class Frame(object):
             t_profile = np.full(self.tchans, t_profile)
         else:
             raise TypeError('t_profile is not a function, array, or float.')
-        t_profile_tt = np.meshgrid(restricted_fs, t_profile)[1]
+        _, t_profile_tt = np.meshgrid(restricted_fs, t_profile)
 
-        # Handle path
+        # Handle path. Generate one extra time sample for freq smearing
+        # calculations
+        tchans_eff = self.tchans
+        if doppler_smearing:
+            tchans_eff += 1
         if callable(path):
             # Average using integration to get a better position in frequency
             # direction
             if integrate_path:
                 new_ts = np.linspace(0,
-                                     self.tchans * self.dt,
-                                     self.tchans * t_subsamples)
+                                     tchans_eff * self.dt,
+                                     tchans_eff * t_subsamples,
+                                     endpoint=False)
                 f = path(new_ts)
                 if not isinstance(f, np.ndarray):
-                    f = np.repeat(f, self.tchans * t_subsamples)
-                integrated_f = np.mean(np.reshape(f, (self.tchans,
+                    f = np.repeat(f, tchans_eff * t_subsamples)
+                integrated_f = np.mean(np.reshape(f, (tchans_eff,
                                                       t_subsamples)),
                                        axis=1)
                 path = integrated_f
             else:
-                path = path(self.ts)
+                ts = self.ts
+                if doppler_smearing:
+                    ts = np.append(self.ts, self.ts[-1] + self.dt)
+                path = path(ts)
         elif isinstance(path, (list, np.ndarray)):
             path = np.array(path)
             if path.shape != self.ts.shape:
-                raise ValueError('Shape of path array is {0} != {1}.'
-                                 .format(path.shape, self.ts.shape))
+                raise ValueError(f'Shape of path array is {path.shape} '
+                                 f'!= {self.ts.shape}.')
+            elif doppler_smearing and len(path) != self.tchans + 1:
+                raise ValueError(f'To Doppler smear power, must provide'
+                                 f'path array with {self.tchans + 1} values')
         elif isinstance(path, (int, float)):
-            path = np.full(self.tchans, path)
+            path = np.full(tchans_eff, path)
         else:
             raise TypeError('path is not a function, array, or float.')
-        path_tt = np.meshgrid(restricted_fs, path)[1]
+        # Ensure that path f_centers are the right length
+        _, path_tt = np.meshgrid(restricted_fs, path[:self.tchans])
+        
+        if doppler_smearing:
+            dpath = np.diff(path) / smearing_subsamples
+            _, dpath_tt = np.meshgrid(restricted_fs, dpath)
 
         # Handle bandpass profile
+        if bp_profile is None:
+            bp_profile = 1
         if callable(bp_profile):
             bp_profile = bp_profile(restricted_fs)
         elif isinstance(bp_profile, (list, np.ndarray)):
@@ -588,9 +753,17 @@ class Frame(object):
             bp_profile = np.full(restricted_fs.shape, bp_profile)
         else:
             raise TypeError('bp_profile is not a function, array, or float.')
-        bp_profile_ff = np.meshgrid(bp_profile, self.ts)[0]
+        bp_profile_ff, _ = np.meshgrid(bp_profile, self.ts)
 
-        signal = t_profile_tt * f_profile(ff, path_tt) * bp_profile_ff
+        # Create signal, adding multiple copies for Doppler smearing case
+        if doppler_smearing:
+            signal = np.zeros(ff.shape)
+            for _ in range(smearing_subsamples):
+                signal += (t_profile_tt * f_profile(ff, path_tt) 
+                           / smearing_subsamples * bp_profile_ff)
+                path_tt += dpath_tt
+        else:
+            signal = t_profile_tt * f_profile(ff, path_tt) * bp_profile_ff
 
         if integrate_f_profile:
             signal = np.mean(np.reshape(signal, (self.tchans,
@@ -610,7 +783,8 @@ class Frame(object):
                             drift_rate,
                             level,
                             width,
-                            f_profile_type='gaussian'):
+                            f_profile_type='sinc2',
+                            doppler_smearing=False):
         """
         A wrapper around add_signal() that injects a constant intensity,
         constant drift_rate signal into the frame.
@@ -625,8 +799,14 @@ class Frame(object):
             Signal intensity
         width : astropy.Quantity
             Signal width in frequency units
-        f_profile_type : str
+        f_profile_type : str, optional
             Can be 'box', 'sinc2', 'gaussian', 'lorentzian', or 'voigt', based on the desired spectral profile
+        doppler_smearing : bool, optional
+            Option to numerically "Doppler smear" spectral power over 
+            frequency bins. At time t, averages drift_rate / frame.unit_drift_rate 
+            copies of the signal centered at evenly spaced center frequencies between 
+            times t and t+1. This causes the effective drop in power when 
+            the signal crosses multiple bins.
 
         Returns
         -------
@@ -640,14 +820,15 @@ class Frame(object):
         start_index = self.get_index(f_start)
 
         # Calculate the bounding box, to optimize signal insertion calculation
+        px_width_offset = 2 * width / self.df
         if drift_rate < 0:
-            px_width_offset = -2 * width / self.df
-        else:
-            px_width_offset = 2 * width / self.df
+            px_width_offset = -px_width_offset
         px_drift_offset = self.dt * (self.tchans - 1) * drift_rate / self.df
+        if doppler_smearing:
+            px_drift_offset += drift_rate * self.dt / self.df
 
-        bounding_start_index = start_index + int(np.floor(-px_width_offset))
-        bounding_stop_index = start_index + int(np.ceil(px_drift_offset + px_width_offset))
+        bounding_start_index = start_index + int(-px_width_offset)
+        bounding_stop_index = start_index + int(px_drift_offset + px_width_offset)
 
         bounding_min_index = max(min(bounding_start_index, bounding_stop_index), 0)
         bounding_max_index = min(max(bounding_start_index, bounding_stop_index), self.fchans)
@@ -665,13 +846,15 @@ class Frame(object):
             f_profile = f_profiles.box_f_profile(width)
         else:
             raise ValueError('Unsupported f_profile for constant signal!')
-
+        
         return self.add_signal(path=paths.constant_path(f_start, drift_rate),
                                t_profile=t_profiles.constant_t_profile(level),
                                f_profile=f_profile,
                                bp_profile=bp_profiles.constant_bp_profile(level=1),
                                bounding_f_range=(self.get_frequency(bounding_min_index),
-                                                 self.get_frequency(bounding_max_index)))
+                                                 self.get_frequency(bounding_max_index)),
+                               doppler_smearing=doppler_smearing,
+                               smearing_subsamples=int(np.ceil(drift_rate / self.unit_drift_rate)))
 
     def get_index(self, frequency):
         """
@@ -687,7 +870,7 @@ class Frame(object):
 
     def get_intensity(self, snr):
         """
-        Calculates intensity from SNR, based on estimates of the noise in the
+        Calculate intensity from SNR, based on estimates of the noise in the
         frame.
 
         Note that there must be noise present in the frame for this to make
@@ -708,11 +891,21 @@ class Frame(object):
             raise ValueError('You must add noise in the image to return SNR!')
         return intensity * np.sqrt(self.tchans) / self.noise_std
 
-    def get_drift_rate(self, start_index, end_index):
-        return (end_index - start_index) * self.df / (self.tchans * self.dt)
+    def get_drift_rate(self, start_index, stop_index):
+        return (stop_index - start_index) * self.df / (self.tchans * self.dt)
 
     def get_info(self):
         return vars(self)
+    
+    def get_params(self):
+        return {
+            'fchans': self.fchans,
+            'tchans': self.tchans,
+            'df': self.df, 
+            'dt': self.dt, 
+            'fch1': self.fch1, 
+            'ascending': self.ascending
+        }
 
     def get_data(self, use_db=False):
         if use_db:
@@ -737,19 +930,76 @@ class Frame(object):
     def add_metadata(self, new_metadata):
         self.update_metadata(new_metadata)
 
-    def render(self, use_db=False):
-        # Display frame data in waterfall format
-        plt.imshow(self.get_data(use_db=use_db),
-                   aspect='auto',
-                   interpolation='none')
-        plt.colorbar()
-        plt.xlabel('Frequency (px)')
-        plt.ylabel('Time (px)')
+    def render(self, use_db=False, cb=True):
+        """
+        Display frame data in waterfall format.
+        
+        Parameters
+        ----------
+        use_db : bool
+            Whether to convert data to dB
+        cb : bool
+            Whether to display colorbar
+        """ 
+        frame_utils.render(self, use_db=use_db, cb=cb)
 
     def bl_render(self, use_db=True):
         self._update_waterfall()
         self.waterfall.plot_waterfall(logged=use_db)
+        
+    def plot(self, use_db=False, cb=True):
+        """
+        Wrapper for render().
+        """
+        self.render(use_db=use_db, cb=cb)
+        
+    def bl_plot(self, use_db=True):
+        """
+        Wrapper for bl_render().
+        """
+        self.bl_render(use_db=use_db)
+        
+    def get_slice(self, l, r):
+        """
+        Slice frame data with left and right index bounds.
+    
+        Parameters
+        ----------
+        l : int
+            Left bound
+        r : int
+            Right bound
 
+        Returns
+        -------
+        s_fr : Frame
+            Sliced frame
+        """
+        return frame_utils.get_slice(self, l, r)
+        
+    def integrate(self, axis='t', mode='mean', normalize=False):
+        """
+        Integrate along either time ('t', 0) or frequency ('f', 1) axes, to create 
+        spectra or time series data. Mode is either 'mean' or 'sum'.
+    
+        Parameters
+        ----------
+        data : Frame, or 2D ndarray
+            Input frame or Numpy array
+        axis : int or str
+            Axis over which to integrate; time ('t', 0) or frequency ('f', 1)
+        mode : str
+            Integration mode, 'mean' or 'sum'
+        normalize : bool
+            Whether to normalize integrated array to mean 0, std 1
+            
+        Returns
+        -------
+        output : ndarray
+            Integrated product
+        """
+        return frame_utils.integrate(self, axis=axis, mode=mode, normalize=normalize)
+        
     def _update_waterfall(self, filename=None, max_load=1):
         # If entirely synthetic, base filterbank structure on existing sample data
         if self.waterfall is None:
@@ -839,6 +1089,17 @@ class Frame(object):
         """
         self._update_waterfall()
         return self.waterfall
+    
+    def check_waterfall(self):
+        """
+        If an associated Waterfall object exists, update and return it. Otherwise,
+        return None. Useful to chain with setigen.Frame.from_data() if manipulating
+        completely synthetic data.
+        """
+        if self.waterfall is None:
+            return None
+        else:
+            return self.get_waterfall()
 
     def save_fil(self, filename, max_load=1):
         """
@@ -891,3 +1152,43 @@ class Frame(object):
         with open(filename, 'rb') as f:
             frame = pickle.load(f)
         return frame
+
+    
+def params_from_backend(obs_length=300, 
+                        sample_rate=3e9, 
+                        num_branches=1024,
+                        fftlength=1048576,
+                        int_factor=51):
+    """
+    Return frame parameters calculated from data backend characteristics.
+
+    Parameters
+    ----------
+    obs_length : float, optional
+        Length of observation in seconds
+    sample_rate : float, optional
+        Physical sample rate, in Hz, for collecting real voltage data
+    num_branches : int, optional
+        Number of PFB branches. Note that this corresponds to `num_branches / 2` coarse channels.
+    fftlength : int, optional
+        FFT length to be used in fine channelization
+    int_factor : int, optional
+        Integration factor used in fine channelization. Determines tchans.
+
+    Returns
+    -------
+    param_dict : dict
+        Dictionary of parameters
+    """
+    chan_bw = sample_rate / num_branches
+    df = chan_bw / fftlength
+
+    dt = int_factor / df
+    tchans = int(obs_length / dt)
+
+    param_dict = {
+        'tchans': tchans,
+        'df': df, 
+        'dt': dt
+    }
+    return param_dict
